﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;

namespace Common
{
    public class WebHelper
    {
        private const string ProxyUrl =
            "http://dev.kuaidaili.com/api/getproxy/?orderid=956699370516392&num=100&b_pcchrome=1&b_pcie=1&b_pcff=1&protocol=1&method=2&an_an=1&an_ha=1&sep=1";

        public readonly WebClient Web = new WebClient();
        private readonly Queue<KeyValuePair<string, int>> _proxyPairs = new Queue<KeyValuePair<string, int>>();
        private int _tryTimes;

        public WebHelper()
        {
            Web.Encoding = Encoding.UTF8;
        }

        public WebHelper(Encoding encoding)
        {
            Web.Encoding = encoding;
        }

        public Encoding Encoding
        {
            set { Web.Encoding = value; }
        }

        /// <summary>
        ///     下载请求的资源
        /// </summary>
        /// <param name="url">URL</param>
        /// <param name="useProxy">是否使用代理</param>
        /// <returns></returns>
        public string DownloadString(string url, bool useProxy = false)
        {
            if (useProxy)
            {
                InitWebProxy();
            }
            else
            {
                Web.Proxy = null;
            }
            try
            {
                return Web.DownloadString(url);
            }
            catch (WebException e)
            {
                if (e.Message.Contains("404") || e.Status == WebExceptionStatus.ConnectFailure ||
                    e.Status == WebExceptionStatus.ProtocolError || _tryTimes == 2)
                {
                    _tryTimes = 0;
                    return null;
                }

                _tryTimes++;
                return DownloadString(url, useProxy);
            }
        }

        /// <summary>
        ///     下载请求的资源(资源采用Gzip压缩)
        /// </summary>
        /// <param name="url">URL</param>
        /// <param name="useProxy">是否使用代理</param>
        /// <returns></returns>
        public string DownloadGzipString(string url, Encoding encoding)
        {
            Web.Headers.Add("Accept-Encoding", "gzip");
            try
            {
                return encoding.GetString(ZipHelper.GzipDecompress(Web.DownloadData(url)));
            }
            catch (WebException e)
            {
                if (e.Message.Contains("404") || e.Status == WebExceptionStatus.ConnectFailure ||
                    e.Status == WebExceptionStatus.ProtocolError || _tryTimes == 2)
                {
                    _tryTimes = 0;
                    return null;
                }

                _tryTimes++;
                return DownloadGzipString(url, encoding);
            }
            finally
            {
                Web.Headers.Remove("Accept-Encoding");
            }
        }

        /// <summary>
        ///     将指定的字符串上载到指定的资源
        /// </summary>
        /// <param name="address">地址</param>
        /// <param name="data">参数</param>
        /// <param name="useProxy">是否使用代理</param>
        /// <returns></returns>
        public string UploadString(string address, string data, bool useProxy = false)
        {
            Web.Headers.Add("Content-Type", "application/x-www-form-urlencoded");
            if (useProxy)
            {
                InitWebProxy();
            }
            else
            {
                Web.Proxy = null;
            }

            try
            {
                return Web.UploadString(address, "POST", data);
            }
            catch
            {
                if (_tryTimes == 2)
                {
                    _tryTimes = 0;
                    return null;
                }

                _tryTimes++;
                return UploadString(address, data);
            }
        }

        public void InitWebProxy()
        {
            if (_proxyPairs.Count == 0)
            {
                AddProxy();
            }
            KeyValuePair<string, int> proxy = _proxyPairs.Dequeue();
            Web.Proxy = new WebProxy(proxy.Key, proxy.Value);
        }

        private void AddProxy()
        {
            string h = DownloadString(ProxyUrl);
            if (string.IsNullOrWhiteSpace(h) || h.Contains("ERROR"))
            {
                //请求太频繁
                Console.Write("重获代理 ");
                Thread.Sleep(60000);
                AddProxy();
            }
            else
            {
                string[] ps = h.Split(new[] {"\r\n", ":"}, StringSplitOptions.None);
                int count = ps.Count();

                for (int i = 0; i < count; i += 2)
                {
                    _proxyPairs.Enqueue(new KeyValuePair<string, int>(ps[i], int.Parse(ps[i + 1])));
                }
            }
        }
    }
}